
import bpy
import os
import shutil
import zipfile
import requests
from bpy.props import StringProperty

from .converters.mesh_converter import MeshConverter
from .converters.scene_converter import SceneConverter
from .converters.material_converter import MaterialConverter


##---------------------------------------------------------------------------------------------------
class ExportFTL(bpy.types.Operator):
    ##-----------------------------------------------------------------------------------------------
    bl_idname       = "export_scene.ftl"
    bl_label        = "Export to 3dverse"
    bl_description  = "Export blender project to 3dverse (scene, meshes, materials)"
    bl_category     = "Export"
    bl_options      = {'REGISTER', 'PRESET'}

    default_folder_id   = "my_folder_id"
    default_api_key     = "my_api_key"

    # Operator Properties:
    # icon="FILE_FOLDER"
    folder_id : StringProperty(name="Folder ID", 
                               description="Folder ID to upload the converted assets to", 
                               default=default_folder_id, 
                               maxlen=36)
    
    # icon="KEY_HLT"
    api_key : StringProperty(name="API Key", 
                             description="API Key with write permission", 
                             default=default_api_key, 
                             maxlen=80)
    
    ##-----------------------------------------------------------------------------------------------       
    def invoke(self, context, _event):
        context.window_manager.invoke_props_dialog(self, title="Export to 3dverse", confirm_text="Export to 3dverse")
        return {'RUNNING_MODAL'}
    
    ##-----------------------------------------------------------------------------------------------       
    def execute(self, context):
        if(bpy.app.online_access is False):
            self.report({"ERROR"}, "Turn on online access to export to 3dverse")
            return {'CANCELLED'}

        if(self.folder_id == self.default_folder_id or self.api_key == self.default_api_key):
            self.report({"ERROR_INVALID_INPUT"}, "Please specify a valid API key and folder ID")
            return {'CANCELLED'}
        
        self.init()
        success = self.convert()
        return {'FINISHED'} if success else {'CANCELLED'}
       
    ##-----------------------------------------------------------------------------------------------   
    def init(self):     
       self.mesh_converter_obj      = MeshConverter()
       self.material_converter_obj  = MaterialConverter()
       self.scene_converter_obj     = SceneConverter(self.mesh_converter_obj, self.material_converter_obj,
                                                  bpy.context.scene.name)

    ##-----------------------------------------------------------------------------------------------
    def convert(self):
        scene_uuid          = self.scene_converter_obj.scene_builder.description.uuid
        output_folder_path  = bpy.utils.extension_path_user(__package__, path=scene_uuid, create=True)
        
        self.mesh_converter_obj.convert_meshes(output_folder_path)
        self.scene_converter_obj.convert(output_folder_path)
        self.material_converter_obj.convert_materials(output_folder_path)
        
        package_file_path = f"{output_folder_path}.zip" 
        self.create_package(output_folder_path, package_file_path)

        self.report({"INFO"}, f"Exporting converted assets to 3dverse folder with id: {self.folder_id}.")
        success = self.export_package(package_file_path)
        os.remove(package_file_path)
        shutil.rmtree(output_folder_path)

        return success

    ##-----------------------------------------------------------------------------------------------
    def export_package(self, package_file_path):
        url = f"https://api.3dverse.com/app/v1/folders/{self.folder_id}/packages"

        package = open(package_file_path, 'rb')

        response = requests.put(url, headers={"api_key" : self.api_key}, data=package)

        if(response.ok):
            self.report({"INFO"}, f"Successfully exported to 3dverse.")
            return True
        else:
            self.report({"ERROR"}, f"Failed to export converted assets to 3dverse. {response.status_code} Error: {response.content}")
            return False

    ##-----------------------------------------------------------------------------------------------
    def create_package(self, output_folder_path, package_file_path):
        with zipfile.ZipFile(package_file_path, 'w') as zipf:
            for root, dirs, files in os.walk(output_folder_path):
                for file in files:
                    file_path = os.path.join(root, file)
                    arcname = os.path.relpath(file_path, start=output_folder_path)
                    zipf.write(file_path, arcname)
